from django.dispatch import receiver
from django.db.models.signals import post_save, pre_save, post_delete
from django.urls import reverse
from django.core.mail import send_mail
from django.template.loader import render_to_string
from django.conf import settings
from django.utils import timezone

from guichet.models import Registration, LogEntry, UserManager, Ticket, Info_Sup_Recv_File
from guichet.utils import maj_state_registration, mail_confirm_registration

from paypal.standard.ipn.signals import valid_ipn_received, invalid_ipn_received
from paypal.standard.models import ST_PP_COMPLETED

import re
import os


@receiver(post_save, sender=Registration, dispatch_uid="maj_info_state")
def update_info_stat(sender, instance, raw, **kwargs):
    if raw:
        return
    maj_state_registration(instance)


@receiver(invalid_ipn_received, dispatch_uid="payement_received_invalid")
def payement_received_invalid(sender, **kwargs):
    if sender.flag:
        send_mail(
            "Erreur Paypal with flag : {}".format(sender.flag_info),
            str(sender.__dict__),
            settings.DEFAULT_FROM_EMAIL,
            [admin[1] for admin in settings.ADMINS],
            fail_silently=False,
        )
    else:
        send_mail(
            "Erreur PayPal with status {}".format(sender.payment_status),
            str(sender.__dict__),
            settings.DEFAULT_FROM_EMAIL,
            [admin[1] for admin in settings.ADMINS],
            fail_silently=False,
        )


@receiver(valid_ipn_received, dispatch_uid="payement_received")
def payement_received(sender, **kwargs):
    ipn_obj = sender
    if ipn_obj.payment_status == ST_PP_COMPLETED:
        m = re.search(settings.PAYPAL_INVOICE_PREFIX + "([0-9]+)", ipn_obj.invoice)
        if not m:
            send_mail(
                "Erreur PayPal: invoice introuvable",
                str(ipn_obj.__dict__),
                settings.DEFAULT_FROM_EMAIL,
                [admin[1] for admin in settings.ADMINS],
                fail_silently=False,
            )
            return
        registration = Registration.objects.filter(id=m.group(1))
        if registration.exists():
            registration = registration.first()
        else:
            send_mail(
                "Erreur PayPal: préréservation supprimée",
                m.group(1),
                settings.DEFAULT_FROM_EMAIL,
                [admin[1] for admin in settings.ADMINS],
                fail_silently=False,
            )
            return
        registration.paid = True
        registration.save()  # au cas où …
        registration.payment_method = Registration.PAYMENT_PAYPAL
        registration.payment_date = timezone.now()
        registration.save()

        # vérification des champs (prix, destinataire, devises
        err = ""
        if ipn_obj.mc_currency != "EUR":
            err += " Mauvaise devise ({} au lieu de EUR)".format(ipn_obj.mc_currency)
        if ipn_obj.mc_gross != registration.get_real_amount():
            err += " Mauvais montant ({} {} au lieu de {} EUR)".format(
                ipn_obj.mc_gross, ipn_obj.mc_currency, registration.get_real_amount()
            )
        if registration.ticket.event.paypal_account.no_account:
            err += " Paiement par Paypal désactivé pour cet évènement"
        elif not registration.ticket.online_payment:
            err += " Paiement par Paypal désactivé pour ce billet"
        if ipn_obj.business != registration.ticket.event.paypal_account.email:
            err += " Mauvais destinataire ({} au lieu de {})".format(
                ipn_obj.business, registration.ticket.event.paypal_account.email
            )
        if ipn_obj.test_ipn != 0 and not settings.DEBUG:
            err += " Paiement par la sandbox paypal (pas d'argent reçu)"
        if err != "":
            registration.reason_opposed = Registration.OPPOSED_BY_PAYPAL
            registration.opposed = True
            registration.save()
            c = {
                "err": err,
                "registration": registration,
                "ticket": registration.ticket,
                "event": registration.ticket.event,
                "lien_edit": settings.BASE_URL
                + reverse("registration-edit", args=[registration.ticket.event.id, registration.id]),
            }
            LogEntry(
                user=registration.owner,
                event=registration.ticket.event,
                reason=LogEntry.LOG_PAYPAL_ERROR,
                registration=registration,
                operation="Place {{registration}} Erreur Paypal {} ".format(err),
            ).save()
            send_mail(
                "Erreur PayPal: Mise en opposition lors du paiement par Paypal",
                render_to_string("emails/registration_opposed_paypal_admin.txt", c),
                settings.DEFAULT_FROM_EMAIL,
                [admin[1] for admin in settings.ADMINS]
                + [admin.user.email for admin in UserManager.objects.filter(event=registration.ticket.event)],
                fail_silently=False,
            )
            if not registration.owner.is_anonyme:
                send_mail(
                    "Billetterie INP-net : opposition lors du paiement par Paypal",
                    render_to_string("emails/registration_opposed.txt", c),
                    settings.DEFAULT_FROM_EMAIL,
                    [registration.owner.email],
                    fail_silently=False,
                )
        else:
            LogEntry(
                user=registration.owner,
                event=registration.ticket.event,
                reason=LogEntry.LOG_PAYPAL,
                registration=registration,
                operation="Place {registration} payée via PayPal",
            ).save()
            mail_confirm_registration(registration)
    else:
        payement_received_invalid(sender, **kwargs)


# voir https://stackoverflow.com/questions/16041232/django-delete-filefield#16041527


@receiver(post_delete, sender=Ticket)
def auto_delete_design_on_delete(sender, instance, **kwargs):
    if instance.design:
        if os.path.isfile(instance.design.path):
            os.remove(instance.design.path)


@receiver(pre_save, sender=Ticket)
def auto_delete_design_on_change(sender, instance, **kwargs):
    if not instance.pk:
        return False

    try:
        old_file = Ticket.objects.get(pk=instance.pk).design
    except Ticket.DoesNotExist:
        return False

    new_file = instance.design
    if old_file and not old_file == new_file:
        if os.path.isfile(old_file.path):
            os.remove(old_file.path)


@receiver(post_delete, sender=Info_Sup_Recv_File)
def auto_delete_info_file_on_delete(sender, instance, **kwargs):
    if instance.upload_file:
        if os.path.isfile(instance.upload_file.path):
            os.remove(instance.upload_file.path)


@receiver(pre_save, sender=Info_Sup_Recv_File)
def auto_delete_info_file_on_change(sender, instance, **kwargs):
    if not instance.pk:
        return False

    try:
        old_file = Info_Sup_Recv_File.objects.get(pk=instance.pk).upload_file
    except Info_Sup_Recv_File.DoesNotExist:
        return False

    new_file = instance.upload_file
    if old_file and not old_file == new_file:
        if os.path.isfile(old_file.path):
            os.remove(old_file.path)
